package com.zhilian.online.config;

import com.zhilian.common.redis.service.RedisService;
import com.zhilian.online.uitls.MessageResolver;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.common.serialization.StringSerializer;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.HashSet;
import java.util.List;
import java.util.Properties;
import java.util.Set;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @BelongsProject: smart-cloud-server
 * @BelongsPackage: com.zhilian.online.config
 * @Author: LiYuan
 * @CreateTime: 2024-03-31  09:45
 * @Description: TODO
 * @Version: 1.0
 */
@Component
@Slf4j
public class MqttxConfig {

    /**
     * 路径
     */
    @Value("${mqtt.server.broker}")
    private String broker;

    /**
     * 主题
     */
    @Value("${mqtt.server.topic}")
    private String topic;

    /**
     * 客户端ID
     */
    @Value("${mqtt.server.clientId}")
    private String clientId;

    /**
     * 用户名
     */
    @Value("${mqtt.server.username}")
    private String username;

    /**
     * 密码
     */
    @Value("${mqtt.server.password}")
    private String password;

    /**
     * qos
     */
    @Value("${mqtt.server.qos}")
    private Integer qos;

    /**
     * redis服务
     */
    @Autowired
    private RedisService redisService;
    /**
     * kafka订阅topic
     */
    private String topicName = "vehicle-topic";

    /**
     * kafka服务
     */
    @Value("${spring.kafka.producer.bootstrap-servers}")
    private String bootstrap_servers;

    /**
     * 线程池
     */
    private ExecutorService executorService = Executors.newSingleThreadExecutor();

    /**
     * kafka服务
     */
    private KafkaProducer<String, String> kafkaProducer;




    @PostConstruct
    public void initMqtt() {
        log.info("mqttx连接中......");

        try {
            MqttClient mqttClient = new MqttClient(broker, clientId, new MemoryPersistence());

            MqttConnectOptions mqttConnectOptions = new MqttConnectOptions();

            mqttConnectOptions.setUserName(username);
            mqttConnectOptions.setPassword(password.toCharArray());

            //链接超时
            mqttConnectOptions.setConnectionTimeout(10);
            //心跳检测
            mqttConnectOptions.setKeepAliveInterval(60);
            mqttClient.connect(mqttConnectOptions);
            log.info("mqtt{}服务连接成功", broker);

            //回调
            mqttClient.setCallback(new MqttCallback() {
                @Override
                public void connectionLost(Throwable throwable) {
                    log.error("连接断开{}", throwable.getMessage());
                    //链接断开后会尝试重新连接
                    while (mqttClient.isConnected()) {
                        try {
                            //每次重连间隔60秒
                            Thread.sleep(1000 * 5);
                            mqttClient.connect();
                        } catch (Exception e) {
                            log.error("重连失败：" + e.getMessage());
                        }
                    }
                }

                @Override
                public void messageArrived(String s, MqttMessage mqttMessage) throws Exception {
                    log.info("消息到达,接受消息主题{},接受消息Qos{},接受消息内容{}", topic, mqttMessage.getQos(), new String(mqttMessage.getPayload()));

//                    HashSet<String> strings = new HashSet<>();
//                    strings.add("malfunction");
//                    strings.add("event-storage");
//                    strings.add("geofence");
//
//                    redisService.setCacheSet("vehicle-event:" + "VIN123456789DIJE4",strings);
//                    Set<Object> cacheSet = redisService.getCacheSet("vehicle-event:" + "VIN123456789DIJE4");
//
//                    if (redisService.hasKey("vehicle-event:" + "VIN123456789DIJE4")){
//                        //有事件
//                        Set<Object> cacheSet1 = redisService.getCacheSet("vehicle-event:" + "VIN123456789DIJE4");
//                        Set<String> events = cacheSet1.stream().map(item -> {
//                            return String.valueOf(item);
//                        }).collect(Collectors.toSet());
//                        if (events.contains("geofence")){
//                            //包含围栏事件
//
//                        }else {
//                            //不包含围栏事件
//                            redisService.setCacheSet("vehicle-event:" + "VIN123456789DIJE4","geofence");
//                            redisService.deleteCacheSet("vehicle-event:" + "VIN123456789DIJE4");
//                            events.add("geofence");
//                            redisService.setCacheSet("vehicle-event:" + "VIN123456789DIJE4",events);
//                        }
//                    }

                    //将接受到的车辆报文存储到kafka中
                    executorService.execute(() -> {
                        executorTask(mqttMessage);
                    });
//                    executorTask(mqttMessage);

                }

                @Override
                public void deliveryComplete(IMqttDeliveryToken iMqttDeliveryToken) {
                    log.info("消息{}发送成功" + iMqttDeliveryToken.isComplete());
                }
            });

            mqttClient.subscribe(topic, qos);

        } catch (MqttException e) {
            throw new RuntimeException(e);
        }


    }





    public void executorTask(MqttMessage mqttMessage){
        //解析得出原始报文String
        String sourceMsg = new String(mqttMessage.getPayload());
        //对原始报文进行解析
        String parseMsg = MessageResolver.parseMsg(sourceMsg);

        //截取原始报文获得车辆VIN码
        String vin = parseMsg.substring(0, 17);

        log.info("当前车辆VIN码:{}",vin);


        //查询该vin码是否为我们的车
        List<Object> cacheList = redisService.getCacheList("our-car");
        List<String> carList = cacheList.stream().map(item -> {
            return String.valueOf(item);
        }).collect(Collectors.toList());
        if (carList.contains(vin)){
            //将车辆vin码存入redis中,用于判断车辆是否处于上线状态
            redisService.setCacheObject("online-vehicle",vin,60L, TimeUnit.SECONDS);

            //发送rabbitMQ信息修改车辆上线状态

            String string = mqttMessage.toString();
            //将车辆报文发送到kafka
            sendMsg(new String(mqttMessage.getPayload()));


        }else {
            //不是我们的车
            log.info("车辆{}不是我们的车",vin);
        }
    }

    /**
     * @return
     * @Description: 初始化kafka服务
     */
    @PostConstruct
    public KafkaProducer<Object, Object> initKafkaProducer() {
        Properties properties = new Properties();
        properties.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrap_servers);
        properties.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringSerializer");
        properties.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringSerializer");
        this.kafkaProducer = new KafkaProducer<>(properties);
        return new KafkaProducer<>(properties);
    }

    /**
     * @param msg
     * @Description: 发送消息
     */
    private void sendMsg(String msg) {
        kafkaProducer.send(new ProducerRecord<>(topicName, msg));
        log.info("Kafka在topic:{}中发送消息{}", topicName, msg);
    }


}
