package com.zhilian.online.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhilian.common.core.domain.Result;
import com.zhilian.common.core.utils.uuid.IdUtils;
import com.zhilian.common.redis.service.RedisService;
import com.zhilian.online.config.RabbitConfig;
import com.zhilian.online.constans.OnlineConstants;
import com.zhilian.online.domain.Gather;
import com.zhilian.online.domain.VehicleAccount;
import com.zhilian.online.mapper.OnlineLoadCenterMapper;
import com.zhilian.online.service.OnlineLoadCenterService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.ZSetOperations;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @version 1.0
 * @Author LiYuan
 * @description 车辆负载中心服务层
 * @date 2024/3/29  20:42
 */
@Service
@Slf4j
public class OnlineLoadCenterServiceImpl extends ServiceImpl<OnlineLoadCenterMapper, VehicleAccount> implements OnlineLoadCenterService {

    /**
     * 车辆负载中心数据层
     */
    @Autowired
    private OnlineLoadCenterMapper onlineLoadCenterMapper;

    /**
     * Redis服务
     */
    @Autowired
    private RedisService redisService;

    /**
     * RabbitMQ服务
     */
    @Autowired
    private RabbitTemplate rabbitTemplate;

    /**
     * @description: 车辆申请注解令牌接口, 仅限内网访问
     * @author: LiYuan
     * @param: vehicle
     * @return: Result<OnlineAccount>
     **/
    @Override
    public Result<String> applyForReg(Gather gather) {

        //生成一次性令牌
        String token = IdUtils.fastSimpleUUID();

        //将令牌信息缓存到Redis中
        redisService.setCacheObject(OnlineConstants.ONLINE_TOKEN_PREFIX+gather.getClientId(),token, OnlineConstants.ONLINE_TOKEN_EXPIRE, TimeUnit.SECONDS);

        //将令牌信息返回客户端
        return Result.success(token);
    }


    /**
     * @description: 节点上线
     * @author: LiYuan
     * @param: Gather
     * @return: Result
     **/
    @Override
    public Result regGather(Gather gather) {
        //判断登录令牌是否过期,一致
        if (!redisService.hasKey(OnlineConstants.ONLINE_TOKEN_PREFIX + gather.getClientId())) {
            return Result.error("令牌已过期");
        }
        String token = redisService.getCacheObject(OnlineConstants.ONLINE_TOKEN_PREFIX + gather.getClientId());
        if (!token.equals(gather.getToken())){
            return Result.error("令牌错误");
        }

        //为该节点创建负载均衡缓存
        redisService.setCacheZsetValue(OnlineConstants.GATHER_LOAD_CONTROL + gather.getClientId(), JSON.toJSONString(gather), 0.0);

        //向RabbitMQ||RocketMQ发送30s延迟消息,确保后续节点上线
        rabbitTemplate.convertAndSend(RabbitConfig.DELAY_EXCHANGE_NAME, RabbitConfig.DELAY_ROUTING_KEY, JSON.toJSONString(gather));

        return Result.success("节点上线");
    }

    /**
     * 车辆申请上线收集节点
     * @author: LiYuan
     * @param: vin
     * @return gather
     */
    @Override
    public Gather applyForConnectToGather(String vin) {
        //判断车辆是否是我们的车
        List<Object> cacheList = redisService.getCacheList("our_car");
        List<String> list = cacheList.stream().map(item -> {
            return String.valueOf(item);
        }).collect(Collectors.toList());
        if (!list.contains(vin)){
            throw new RuntimeException("车辆未登记");
        }
        //获取负载最少的车辆进行链接
        ZSetOperations.TypedTuple cacheZsetMin = redisService.getCacheZsetMin(OnlineConstants.GATHER_LOAD_CONTROL);
        Gather gather = JSON.parseObject((String) cacheZsetMin.getValue(), Gather.class);
        //存放节点车辆信息
        redisService.setCacheObject(OnlineConstants.ONLINE_VEHICLE+vin,gather.getClientId());
        //发送延迟队列确定车辆上线
        rabbitTemplate.convertAndSend(RabbitConfig.DELAY_EXCHANGE_FOR_CAR, RabbitConfig.DELAY_ROUTING_FOR_CAR,vin);

        return gather;
    }

    /**
     * @description:车辆下线接口
     * @author: LiYuan
     * @param: vin
     * @return: Result
     **/
    @Override
    public void applyForDisconnectToGather(String vin) {

    }


}
