package com.zhilian.resolver.service.impl.eventGeofence;

import com.zhilian.common.resolver.domain.ResolverReportData;
import com.zhilian.resolver.GeoFence;
import com.zhilian.resolver.service.ResolverEventService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;

import static java.lang.Double.parseDouble;

/**
 * @ClassName GeofenceEventServiceImpl
 * @Description 电子围栏事件
 * @Author Can.J
 * @Date 2024/4/4 10:25
 */
@Slf4j
@Service("geofence")
public class GeofenceEventServiceImpl implements ResolverEventService {

    @Override
    public void execute(ResolverReportData resolverReportData) {
        log.info("电子围挡执行");
        String vin = resolverReportData.getVin();
        log.info("vin :" + vin);
        // 经度
        double pointLng = parseDouble(resolverReportData.getLongitude());
        // 维度
        double pointLat = parseDouble(resolverReportData.getLatitude());

        // 调用电子围栏信息
        GeoFence geoFence = new GeoFence();
        // 39.958897,116.179075;    39.950476,116.330823;    39.865149,116.290311;     39.881485,116.083631
        // 廊坊经度：116.683752  维度：39.538047
        geoFence.setLongitudeAndLatitude("39.526907,116.695848;39.522727,116.686888;39.527536,116.708916;39.522847,116.684066");

        String[] split = geoFence.getLongitudeAndLatitude().split(";");

        // 判断点是否在电子围栏内
        boolean withinFence = isWithinFence(pointLat, pointLng, Arrays.asList(split));

        if (withinFence) {
            log.info("在围栏内");
        } else {
            log.error("不在围栏内");
        }

        log.info("电子围挡结束");
    }

    @Override
    public String getEventName() {
        return "geofence";
    }

    /**
     * 判断点是否在矩形内（射线法）
     *
     * @param pointLat  纬度
     * @param pointLng 经度
     * @param coordinates 矩形顶点坐标数组，格式为：[纬度1,经度1,纬度2,经度2,纬度3,经度3]
     * @return 如果点在矩形内，返回true；否则返回false
     */
    private boolean isWithinFence(double pointLat, double pointLng, List<String> coordinates) {
        if (coordinates.size() < 4) {
            throw new IllegalArgumentException("不符合条件");
        }

        double[] longitudes = new double[4];
        double[] latitudes = new double[4];

        for (int i = 0; i < 4; i++) {
            String[] coordinate = coordinates.get(i).split(",");
            latitudes[i] = parseDouble(coordinate[0]);
            longitudes[i] = parseDouble(coordinate[1]);
        }

        return isPointInRectangle(pointLat, pointLng, latitudes, longitudes);
    }

    /**
     * 判断点是否在矩形内（射线法）
     *
     * @param pointLat  纬度
     * @param pointLng 经度
     * @param latitudes 矩形顶点纬度数组
     * @param longitudes 矩形顶点经度数组
     * @return 如果点在矩形内，返回true；否则返回false
     */
    private boolean isPointInRectangle(double pointLat, double pointLng, double[] latitudes, double[] longitudes) {
        // 根据矩形顶点顺序确定最小和最大纬度、经度
        double minLatitude = Arrays.stream(latitudes).min().orElseThrow();
        double maxLatitude = Arrays.stream(latitudes).max().orElseThrow();
        double minLongitude = Arrays.stream(longitudes).min().orElseThrow();
        double maxLongitude = Arrays.stream(longitudes).max().orElseThrow();

        return pointLat >= minLatitude && pointLat <= maxLatitude &&
                pointLng >= minLongitude && pointLng <= maxLongitude;
    }
}
